#!/usr/bin/env python

# Copyright (c) 2017  University of Houston

import argparse
import logging
import sys

logging.basicConfig(level = logging.INFO)

import geni.aggregate.vts as VTSAM
import geni.minigcf.config as CFG
import geni.rspec.vts as VTS
import geni.util

import uhgeni.ssh

CFG.HTTP.TIMEOUT = 90

def parse_args ():
  parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
  parser.add_argument("--uhexpbranch", dest="uhexpbranch", type=str, default="default")
  parser.add_argument("--uhexpurl", dest="uhexpurl", type=str, default="https://bitbucket.org/uh-netlab/uhexp")
  parser.add_argument("--site", dest="site", type=str, default="vts-clemson")
  parser.add_argument("--slice", dest="slice", type=str)
  parser.add_argument("--project", dest="project", type=str, default=None)
  parser.add_argument("--context-path", dest="context_path", type=str, default=None)
  parser.add_argument("--config-path", dest="config_path", type=str, default="./expconfig")
  parser.add_argument("--delete", dest="delete", action="store_true", default=False)

  return parser.parse_args()


def conNodes (r, uhexpbranch, uhexpurl):
  node_list = []
  client = r.Container(VTS.Image("uh.net-client"), "client")
  client.ram = 48
  client.HgMount("uhexp", uhexpurl, "/uhexp", uhexpbranch)
  client.DropboxMount("clientResults", "/exp-data")

  server = r.Container(VTS.Image("uh.nginx"), "server")
  server.ram = 80
  server.image.setImageAttribute("syslog-ip", "10.0.1.200")
  server.HgMount("uhexp", uhexpurl, "/uhexp", uhexpbranch)
  server.DropboxMount("serverResults", "/exp-data")

  syslog = r.Container(VTS.Image("svc.rsyslog"), "syslog")

  monitor = r.Container(VTS.Image("uh.net-client"), "monitor")
  monitor.HgMount("uhexp", uhexpurl, "/uhexp", uhexpbranch)
  monitor.DropboxMount("monitorResults", "/exp-data")

  dp = r.Datapath(VTS.OVSL2Image(), "bridge")

  (dpp0, clientp0) = VTS.connectInternalCircuit(dp, client)
  clientp0.addIPv4Address("10.0.1.1/24")
  (dpp1, serverp0) = VTS.connectInternalCircuit(dp, server, VTS.DelayInfo(25))
  serverp0.addIPv4Address("10.0.1.3/24")
  (dpp2, monitorp0) = VTS.connectInternalCircuit(dp, monitor)
  dp.image.setMirror(dpp2)

  (_, lif) = VTS.connectInternalCircuit(dp, syslog)
  lif.addIPv4Address("10.0.1.200/24")
  return 

def buildRequest (opts):
  r = VTS.Request()
  conNodes(r, opts.uhexpbranch, opts.uhexpurl)
  return r

if __name__ == '__main__':
  opts = parse_args()

  context = geni.util.loadContext(path = opts.context_path, key_passphrase=True)
  if opts.project:
    context.project = opts.project

  am = VTSAM.name_to_aggregate()[opts.site]
  if opts.delete:
    logging.info("Deleting existing sliver")
    am.deletesliver(context, opts.slice)

  logging.info("Building request")
  r = buildRequest(opts)
  r.writeXML("request.xml")
  logging.info("Request saved at request.xml")

  logging.info("Calling createsliver")
  m = am.createsliver(context, opts.slice, r)
  m.writeXML("manifest.xml")
  logging.info("Manifest saved at manifest.xml")

  with open("topo.dot", "w+") as f:
    f.write(geni.util.builddot([m]))
  logging.info("Dotfile saved at topo.dot")

  uhgeni.ssh.writeSliceConfig(opts.slice, m, path = opts.config_path)
  logging.info("ssh config file saved at %s" % (opts.config_path))
