# Copyright (c) 2017  University of Houston

# Reference example of fully orchestrated experiment using UHEXP tools

import uhexp.client as XC
import uhexp.util

######################
# Orchestrator Setup #
######################

logh = uhexp.util.getLog("ref-exp.log")

node_builder = XC.ClientFactory(logh, ssh_cfg_path = "expconfig", manifests = ["manifest.xml"])


# Get client objects for each node, bound to the required infrastructure specialization (VTS, in this case)
server = node_builder.build("server")
monitor = node_builder.build("monitor")
client = node_builder.build("client")
nodes = [server, monitor, client]

# We need to install uhexp everywhere
for node in nodes:
  XC.Base(node).install()

# Wait for install to finish
XC.waitForJobs()


####################
# Experiment Setup #
####################

# Get handles to the orchestration tools for the apps we need to use, bound
# to the nodes we're going to use them on
mon_tdmp = XC.TCPDump(monitor)
srv_nginx = XC.Nginx(server)
srv_dd = XC.DD(server)
cl_curl = XC.Curl(client)

# Start all the required installs
tkl = []
tkl.append(mon_tdmp.install())
tkl.append(srv_nginx.install())
tkl.append(srv_dd.install())
tkl.append(cl_curl.install())

# Wait for the installs to complete
XC.waitForJobs()

# Utility function to end script if a token is marked failed
uhexp.util.checkAndDie(tkl)

# Start nginx
srv_nginx.start()

# Collect OS info from all the nodes
for node in nodes:
  XC.UHGetConf(node).run()
XC.waitForJobs()

# Do required setup on nginx to make data for download
srv_dd.run_block(ofpath = "/www-root/bigfile.bin", ifpath = "/dev/urandom", blocksize = 1024*1024, count = 100)


##################
# Experiment Run #
##################

# Start tcpdump - does not return a job token
mon_tdmp.start(interface = "eth1", snaplen = 200, ofpath = "/exp-data/monitor.pcap")

# Run the transfer
ctkn = cl_curl.run("http://10.0.1.3/bigfile.bin")

# Wait for the transfer(s) to finish
XC.waitForJobs()

# Stop tcpdump
mon_tdmp.stop()


#######################
# Get Experiment Data #
#######################

server.downloadData(local_root = "./exp-data/")
monitor.downloadData(local_root = "./exp-data/")
client.downloadData(local_root = "./exp-data/")

